/** 
 * Project: GuitFun PM Project 
 * Octavian Valeriu Crintea, 333CA
 */

/**
 * Acknowledgements:
 * Project: AVR ATtiny USB Tutorial at http://codeandlife.com/
 * Author: Joonas Pihlajamaa, joonas.pihlajamaa@iki.fi
 * Inspired by V-USB example code by Christian Starkjohann
 * Copyright: (C) 2012 by Joonas Pihlajamaa
 * License: GNU GPL v3 (see License.txt)
 */
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>
#include <stdlib.h>

#include "usbdrv.h"

#define F_CPU 16000000L
#include <util/delay.h>

#define USB_LED_OFF 0
#define USB_LED_ON  1
#define USB_DATA_OUT 2
#define USB_DATA_WRITE 3
#define USB_DATA_IN 4

static uchar replyBuf[16] = "Hello, USB!";
static uchar dataReceived = 0, dataLength = 0; // for USB_DATA_IN

// this gets called when custom control message is received
USB_PUBLIC uchar usbFunctionSetup(uchar data[8]) {
	usbRequest_t *rq = (void *)data; // cast data to correct type
	
	static unsigned char my_key = 0;
	
	switch(rq->bRequest) { // custom command is in the bRequest field
    case USB_LED_ON:
		#if 0
		PORTB |= 1; // turn LED on
		#endif
		#if 0
		PORTD |= (1 << PD7);
		#endif
		#if 1
		//my_key = rand() % 3;
		my_key = 0;
		if ((PINB & (1 << PB0)) == 0) {
			my_key |= 1;
		}
		if ((PINB & (1 << PB1)) == 0) {
			my_key |= 2;
		}
		if ((PINB & (1 << PB2)) == 0) {
			my_key |= 4;
		}
		if ((PINB & (1 << PB3)) == 0) {
			my_key |= 8;
		}
		if ((PINB & (1 << PB4)) == 0) {
			my_key |= 16;
		}
		usbMsgPtr = &my_key;
		return sizeof(unsigned char);
		#endif
		return 0;
	case USB_LED_OFF: 
		#if 0
		PORTB &= ~1; // turn LED off
		#endif
		PORTD &= ~(1 << PD7);
		return 0;
    case USB_DATA_OUT: // send data to PC
        usbMsgPtr = replyBuf;
        return sizeof(replyBuf);
	case USB_DATA_WRITE: // modify reply buffer
		replyBuf[7] = rq->wValue.bytes[0];
		replyBuf[8] = rq->wValue.bytes[1];
		replyBuf[9] = rq->wIndex.bytes[0];
		replyBuf[10] = rq->wIndex.bytes[1];
		return 0;
    case USB_DATA_IN: // receive data from PC
		dataLength  = (uchar)rq->wLength.word;
        dataReceived = 0;
		
		if(dataLength  > sizeof(replyBuf)) // limit to buffer size
			dataLength  = sizeof(replyBuf);
			
		return USB_NO_MSG; // usbFunctionWrite will be called now
    }

    return 0; // should not get here
}

// This gets called when data is sent from PC to the device
USB_PUBLIC uchar usbFunctionWrite(uchar *data, uchar len) {
	uchar i;
			
	for(i = 0; dataReceived < dataLength && i < len; i++, dataReceived++)
		replyBuf[dataReceived] = data[i];
		
    return (dataReceived == dataLength); // 1 if we received it all, 0 if not
}

int main() {
	uchar i;

#if 0
	DDRB = 1; // PB0 as output
#endif

	DDRD |= (1 << PD7);
	DDRB &= ~(1 << PB0);
	PORTB |= (1 << PB0);
	DDRB &= ~(1 << PB1);
	PORTB |= (1 << PB1);
	DDRB &= ~(1 << PB2);
	PORTB |= (1 << PB2);
	DDRB &= ~(1 << PB3);
	PORTB |= (1 << PB3);
	DDRB &= ~(1 << PB4);
	PORTB |= (1 << PB4);

    wdt_enable(WDTO_1S); // enable 1s watchdog timer

    usbInit();
	
    usbDeviceDisconnect(); // enforce re-enumeration
    for(i = 0; i<250; i++) { // wait 500 ms
        wdt_reset(); // keep the watchdog happy
        _delay_ms(2);
    }
    usbDeviceConnect();
	
    sei(); // Enable interrupts after re-enumeration
	
    while(1) {
        wdt_reset(); // keep the watchdog happy
        usbPoll();
    }
	
    return 0;
}
